@@ -5,29 +5,23 @@
  *
  * @author  Ben Gardner
  * @license GPL v2+
- *
- * $Id: base_types.h 1313 2008-08-01 18:42:10Z bengardner $
  */
-
 #ifndef BASE_TYPES_H_INCLUDED
 #define BASE_TYPES_H_INCLUDED
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-
 #ifdef WIN32
 
-#include "windows_compat.h"
+#include "../win32/windows_compat.h"
 
 #else /* not WIN32 */
 
+#include "config.h"
+
 #define PATH_SEP    '/'
 
-#if defined HAVE_STDINT_H
-#include <stdint.h>
-#elif defined HAVE_INTTYPES_H
+#define __STDC_FORMAT_MACROS
+
+#if defined HAVE_INTTYPES_H
 #include <inttypes.h>
 #else
 #error "Don't know where int8_t is defined"
@@ -45,22 +39,23 @@ typedef int32_t    INT32;
 typedef uint8_t    UINT8;
 typedef uint16_t   UINT16;
 typedef uint32_t   UINT32;
+typedef uint64_t   UINT64;
 
 #endif   /* ifdef WIN32 */
 
 /* and the good old SUCCESS/FAILURE */
 
-#define SUCCESS     0
-#define FAILURE     -1
+#define SUCCESS    0
+#define FAILURE    -1
 
 
 /* and a nice macro to keep SlickEdit happy */
 
-#define static_inline     static inline
+#define static_inline    static inline
 
 /* and the ever-so-important array size macro */
 #ifndef ARRAY_SIZE
-#define ARRAY_SIZE(x)     (sizeof(x) / sizeof((x)[0]))
+#define ARRAY_SIZE(x)    (sizeof(x) / sizeof((x)[0]))
 #endif
 
 #endif   /* BASE_TYPES_H_INCLUDED */
