@@ -144,8 +144,8 @@ void align_to_column(chunk_t *pc, int column)
       return;
    }
 
-   LOG_FMT(LINDLINE, "%s: %d] col %d on %.*s [%s] => %d\n",
-           __func__, pc->orig_line, pc->column, pc->len, pc->str,
+   LOG_FMT(LINDLINE, "%s: %d] col %d on %s [%s] => %d\n",
+           __func__, pc->orig_line, pc->column, pc->str.c_str(),
            get_token_name(pc->type), column);
 
    int col_delta = column - pc->column;
@@ -219,8 +219,8 @@ void align_to_column(chunk_t *pc, int column)
  */
 void reindent_line2(chunk_t *pc, int column, const char *fcn_name, int lineno)
 {
-   LOG_FMT(LINDLINE, "%s: %d] col %d on %.*s [%s] => %d <called from '%s' line %d\n",
-           __func__, pc->orig_line, pc->column, pc->len, pc->str,
+   LOG_FMT(LINDLINE, "%s: %d] col %d on %s [%s] => %d <called from '%s' line %d\n",
+           __func__, pc->orig_line, pc->column, pc->str.c_str(),
            get_token_name(pc->type), column, fcn_name, lineno);
 
    if (column == pc->column)
@@ -264,8 +264,8 @@ void reindent_line2(chunk_t *pc, int column, const char *fcn_name, int lineno)
          {
             pc->column = min_col;
          }
-         LOG_FMT(LINDLINED, "   set column of '%.*s' to %d (orig %d)\n",
-                 pc->len, pc->str, pc->column, pc->orig_col);
+         LOG_FMT(LINDLINED, "   set column of '%s' to %d (orig %d)\n",
+                 pc->str.c_str(), pc->column, pc->orig_col);
       }
    } while ((pc != NULL) && (pc->nl_count == 0));
 }
@@ -411,6 +411,7 @@ void indent_text(void)
    int                sql_orig_col = 0;
 
    memset(&frm, 0, sizeof(frm));
+   cpd.frame_count = 0;
 
    /* dummy top-level entry */
    frm.pse[0].indent     = 1;
@@ -553,7 +554,7 @@ void indent_text(void)
       {
          if (pc->type == CT_STRING)
          {
-            if ((pc->len > 4) &&
+            if ((pc->len() > 4) &&
                 (xml_indent > 0) &&
                 (pc->str[1] == '<') &&
                 (pc->str[2] == '/'))
@@ -580,12 +581,9 @@ void indent_text(void)
       {
          old_pse_tos = frm.pse_tos;
 
-         /* End anything that drops a level
-          * REVISIT: not sure about the preproc check
-          */
+         /* End anything that drops a level */
          if (!chunk_is_newline(pc) &&
              !chunk_is_comment(pc) &&
-             ((pc->flags & PCF_IN_PREPROC) == 0) &&
              (frm.pse[frm.pse_tos].level > pc->level))
          {
             indent_pse_pop(frm, pc);
@@ -617,7 +615,6 @@ void indent_text(void)
             /* End any assign operations with a semicolon on the same level */
             if (chunk_is_semicolon(pc) &&
                 ((frm.pse[frm.pse_tos].type == CT_IMPORT) ||
-                 (frm.pse[frm.pse_tos].type == CT_THROW) ||
                  (frm.pse[frm.pse_tos].type == CT_USING)))
             {
                indent_pse_pop(frm, pc);
@@ -637,6 +634,7 @@ void indent_text(void)
                 ((pc->type == CT_BRACE_OPEN) ||
                  (pc->type == CT_OC_END) ||
                  (pc->type == CT_OC_SCOPE) ||
+                 (pc->type == CT_OC_PROPERTY) ||
                  chunk_is_semicolon(pc)))
             {
                indent_pse_pop(frm, pc);
@@ -659,9 +657,10 @@ void indent_text(void)
                indent_pse_pop(frm, pc);
             }
 
-            /* a return is ended with a semicolon */
-            if ((frm.pse[frm.pse_tos].type == CT_RETURN) &&
-                chunk_is_semicolon(pc))
+            /* return & throw are ended with a semicolon */
+            if (chunk_is_semicolon(pc) &&
+                ((frm.pse[frm.pse_tos].type == CT_RETURN) ||
+                 (frm.pse[frm.pse_tos].type == CT_THROW)))
             {
                indent_pse_pop(frm, pc);
             }
@@ -700,8 +699,8 @@ void indent_text(void)
 
       if (!chunk_is_newline(pc) && !chunk_is_comment(pc))
       {
-         LOG_FMT(LINDPC, " -=[ %.*s ]=- top=%d %s %d/%d\n",
-                 pc->len, pc->str,
+         LOG_FMT(LINDPC, " -=[ %s ]=- top=%d %s %d/%d\n",
+                 pc->str.c_str(),
                  frm.pse_tos,
                  get_token_name(frm.pse[frm.pse_tos].type),
                  frm.pse[frm.pse_tos].indent_tmp,
@@ -1012,7 +1011,14 @@ void indent_text(void)
             if (chunk_is_newline(prev))
             {
                frm.pse[frm.pse_tos].indent += 2;
-               /* don't change indent of current line */
+
+               if (cpd.settings[UO_indent_ctor_init].n > 0)
+               {
+                  frm.pse[frm.pse_tos].indent     += cpd.settings[UO_indent_ctor_init].n;
+                  frm.pse[frm.pse_tos].indent_tmp += cpd.settings[UO_indent_ctor_init].n;
+                  frm.pse[frm.pse_tos].indent_tab += cpd.settings[UO_indent_ctor_init].n;
+                  indent_column_set(frm.pse[frm.pse_tos].indent_tmp);
+               }
             }
             else
             {
@@ -1034,7 +1040,7 @@ void indent_text(void)
           * after a newline.
           */
          indent_pse_push(frm, pc);
-         frm.pse[frm.pse_tos].indent = pc->column + pc->len;
+         frm.pse[frm.pse_tos].indent = pc->column + pc->len();
 
          if (((pc->type == CT_FPAREN_OPEN) || (pc->type == CT_ANGLE_OPEN)) &&
              ((cpd.settings[UO_indent_func_call_param].b &&
@@ -1096,8 +1102,12 @@ void indent_text(void)
             }
             else
             {
-               if (!chunk_is_comment(next))
+               if (next && !chunk_is_comment(next))
                {
+                  if (next->type == CT_SPACE)
+                  {
+                     next = chunk_get_next_nc(next);
+                  }
                   frm.pse[frm.pse_tos].indent = next->column;
                }
             }
@@ -1113,7 +1123,8 @@ void indent_text(void)
          if (cpd.settings[UO_indent_continue].n != 0)
          {
             frm.pse[frm.pse_tos].indent = continue_indent(pc);
-            indent_column_set(frm.pse[frm.pse_tos].indent);
+            /* the continuation indent applies to the next line, not the current */
+            //indent_column_set(frm.pse[frm.pse_tos].indent);
          }
          frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos].indent;
          frm.paren_count++;
@@ -1131,8 +1142,8 @@ void indent_text(void)
          {
             frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos].indent + indent_size;
             indent_column_set(frm.pse[frm.pse_tos].indent_tmp);
-            LOG_FMT(LINDENT, "%s: %d] assign => %d [%.*s]\n",
-                    __func__, pc->orig_line, indent_column, pc->len, pc->str);
+            LOG_FMT(LINDENT, "%s: %d] assign => %d [%s]\n",
+                    __func__, pc->orig_line, indent_column, pc->str.c_str());
             reindent_line(pc, frm.pse[frm.pse_tos].indent_tmp);
          }
 
@@ -1154,12 +1165,13 @@ void indent_text(void)
             }
             else
             {
-               frm.pse[frm.pse_tos].indent = pc->column + pc->len + 1;
+               frm.pse[frm.pse_tos].indent = pc->column + pc->len() + 1;
             }
             frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos].indent;
          }
       }
-      else if (pc->type == CT_RETURN)
+      else if ((pc->type == CT_RETURN) ||
+               ((pc->type == CT_THROW) && (pc->parent_type == CT_NONE)))
       {
          /* don't count returns inside a () or [] */
          if (pc->level == pc->brace_level)
@@ -1171,45 +1183,11 @@ void indent_text(void)
             }
             else
             {
-               frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 1].indent + pc->len + 1;
+               frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 1].indent + pc->len() + 1;
             }
             frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos - 1].indent;
          }
       }
-      else if (pc->type == CT_THROW)
-      {
-         // Pick up what was just before this.
-         prev = chunk_get_prev(pc);
-         if (pc->parent_type == CT_FUNC_PROTO)
-         {
-            indent_pse_push(frm, pc);
-
-            if ((cpd.settings[UO_indent_func_throw].n != 0) &&
-                ((prev == NULL) || (prev->type == CT_NEWLINE)))
-            {
-               frm.pse[frm.pse_tos].indent = cpd.settings[UO_indent_func_throw].n;
-            }
-            else
-            {
-               frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 1].indent_tmp + indent_size;
-            }
-            frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos].indent;
-            frm.pse[frm.pse_tos].indent_tab = frm.pse[frm.pse_tos].indent;
-
-            indent_column_set(frm.pse[frm.pse_tos].indent_tmp);
-         }
-         else if (pc->parent_type == CT_FUNC_DEF)
-         {
-            if ((cpd.settings[UO_indent_func_throw].n != 0) &&
-                ((prev == NULL) || (prev->type == CT_NEWLINE)))
-            {
-               indent_column_set(cpd.settings[UO_indent_func_throw].n);
-               LOG_FMT(LINDENT, "%s: %d] throw => %d [%.*s]\n",
-                       __func__, pc->orig_line, indent_column, pc->len, pc->str);
-               reindent_line(pc, indent_column);
-            }
-         }
-      }
       else if (pc->type == CT_OC_SCOPE)
       {
          indent_pse_push(frm, pc);
@@ -1259,12 +1237,12 @@ void indent_text(void)
       /**
        * Indent the line if needed
        */
-      if (did_newline && !chunk_is_newline(pc) && (pc->len != 0))
+      if (did_newline && !chunk_is_newline(pc) && (pc->len() != 0))
       {
          pc->column_indent = frm.pse[frm.pse_tos].indent_tab;
 
-         LOG_FMT(LINDENT2, "%s: %d] %d for %.*s\n",
-                 __func__, pc->orig_line, pc->column_indent, pc->len, pc->str);
+         LOG_FMT(LINDENT2, "%s: %d] %d for %s\n",
+                 __func__, pc->orig_line, pc->column_indent, pc->str.c_str());
 
          /**
           * Check for special continuations.
@@ -1282,8 +1260,8 @@ void indent_text(void)
                   cpd.settings[UO_indent_preserve_sql].b)
          {
             reindent_line(pc, sql_col + (pc->orig_col - sql_orig_col));
-            LOG_FMT(LINDENT, "Indent SQL: [%.*s] to %d (%d/%d)\n",
-                    pc->len, pc->str, pc->column, sql_col, sql_orig_col);
+            LOG_FMT(LINDENT, "Indent SQL: [%s] to %d (%d/%d)\n",
+                    pc->str.c_str(), pc->column, sql_col, sql_orig_col);
          }
          else if (((pc->flags & PCF_STMT_START) == 0) &&
                   ((pc->type == CT_MEMBER) ||
@@ -1327,8 +1305,8 @@ void indent_text(void)
          }
          else if (pc->type == CT_PREPROC)
          {
-            LOG_FMT(LINDENT, "%s: %d] pp-indent => %d [%.*s]\n",
-                    __func__, pc->orig_line, indent_column, pc->len, pc->str);
+            LOG_FMT(LINDENT, "%s: %d] pp-indent => %d [%s]\n",
+                    __func__, pc->orig_line, indent_column, pc->str.c_str());
             reindent_line(pc, indent_column);
          }
          else if (chunk_is_paren_close(pc) || (pc->type == CT_ANGLE_CLOSE))
@@ -1359,8 +1337,8 @@ void indent_text(void)
                   }
                }
             }
-            LOG_FMT(LINDENT, "%s: %d] cl paren => %d [%.*s]\n",
-                    __func__, pc->orig_line, indent_column, pc->len, pc->str);
+            LOG_FMT(LINDENT, "%s: %d] cl paren => %d [%s]\n",
+                    __func__, pc->orig_line, indent_column, pc->str.c_str());
             reindent_line(pc, indent_column);
          }
          else if (pc->type == CT_COMMA)
@@ -1370,13 +1348,13 @@ void indent_text(void)
             {
                indent_column_set(frm.pse[frm.pse_tos].pc->column);
             }
-            LOG_FMT(LINDENT, "%s: %d] comma => %d [%.*s]\n",
-                    __func__, pc->orig_line, indent_column, pc->len, pc->str);
+            LOG_FMT(LINDENT, "%s: %d] comma => %d [%s]\n",
+                    __func__, pc->orig_line, indent_column, pc->str.c_str());
             reindent_line(pc, indent_column);
          }
          else if (cpd.settings[UO_indent_func_const].n &&
                   (pc->type == CT_QUALIFIER) &&
-                  (strncasecmp(pc->str, "const", pc->len) == 0) &&
+                  (strncasecmp(pc->text(), "const", pc->len()) == 0) &&
                   ((next == NULL) ||
                    (next->type == CT_BRACED) ||
                    (next->type == CT_BRACE_OPEN) ||
@@ -1387,8 +1365,18 @@ void indent_text(void)
          {
             // indent const - void GetFoo(void)\n const\n { return (m_Foo); }
             indent_column_set(cpd.settings[UO_indent_func_const].n);
-            LOG_FMT(LINDENT, "%s: %d] const => %d [%.*s]\n",
-                    __func__, pc->orig_line, indent_column, pc->len, pc->str);
+            LOG_FMT(LINDENT, "%s: %d] const => %d [%s]\n",
+                    __func__, pc->orig_line, indent_column, pc->str.c_str());
+            reindent_line(pc, indent_column);
+         }
+         else if (cpd.settings[UO_indent_func_throw].n &&
+                  (pc->type == CT_THROW) &&
+                  (pc->parent_type != CT_NONE))
+         {
+            // indent throw - void GetFoo(void)\n throw()\n { return (m_Foo); }
+            indent_column_set(cpd.settings[UO_indent_func_throw].n);
+            LOG_FMT(LINDENT, "%s: %d] throw => %d [%s]\n",
+                    __func__, pc->orig_line, indent_column, pc->str.c_str());
             reindent_line(pc, indent_column);
          }
          else if (pc->type == CT_BOOL)
@@ -1400,19 +1388,19 @@ void indent_text(void)
                if (cpd.settings[UO_indent_first_bool_expr].b)
                {
                   reindent_line(chunk_get_next(frm.pse[frm.pse_tos].pc),
-                                indent_column + pc->len + 1);
+                                indent_column + pc->len() + 1);
                }
             }
-            LOG_FMT(LINDENT, "%s: %d] bool => %d [%.*s]\n",
-                    __func__, pc->orig_line, indent_column, pc->len, pc->str);
+            LOG_FMT(LINDENT, "%s: %d] bool => %d [%s]\n",
+                    __func__, pc->orig_line, indent_column, pc->str.c_str());
             reindent_line(pc, indent_column);
          }
          else
          {
             if (pc->column != indent_column)
             {
-               LOG_FMT(LINDENT, "%s: %d] indent => %d [%.*s]\n",
-                       __func__, pc->orig_line, indent_column, pc->len, pc->str);
+               LOG_FMT(LINDENT, "%s: %d] indent => %d [%s]\n",
+                       __func__, pc->orig_line, indent_column, pc->str.c_str());
                reindent_line(pc, indent_column);
             }
          }
@@ -1432,7 +1420,7 @@ void indent_text(void)
             if (!frm.pse[frm.pse_tos].non_vardef &&
                 (frm.pse[frm.pse_tos].type == CT_BRACE_OPEN))
             {
-               int tmp = indent_column;
+               tmp = indent_column;
                if (cpd.settings[UO_indent_var_def_blk].n > 0)
                {
                   tmp = cpd.settings[UO_indent_var_def_blk].n;
@@ -1442,8 +1430,8 @@ void indent_text(void)
                   tmp += cpd.settings[UO_indent_var_def_blk].n;
                }
                reindent_line(pc, tmp);
-               LOG_FMT(LINDENT, "%s: %d] var_type indent => %d [%.*s]\n",
-                       __func__, pc->orig_line, tmp, pc->len, pc->str);
+               LOG_FMT(LINDENT, "%s: %d] var_type indent => %d [%s]\n",
+                       __func__, pc->orig_line, tmp, pc->str.c_str());
             }
          }
          else
@@ -1481,10 +1469,10 @@ void indent_text(void)
       {
          if (pc->type == CT_STRING)
          {
-            if ((pc->len > 4) &&
+            if ((pc->len() > 4) &&
                 (pc->str[1] == '<') &&
                 (pc->str[2] != '/') &&
-                (pc->str[pc->len - 3] != '/'))
+                (pc->str[pc->len() - 3] != '/'))
             {
                if (xml_indent <= 0)
                {
@@ -1773,11 +1761,11 @@ void indent_preproc(void)
             {
                mult = 1;
             }
-            reindent_line(next, pc->column + pc->len + (pp_level * mult));
+            reindent_line(next, pc->column + pc->len() + (pp_level * mult));
          }
          else if ((cpd.settings[UO_pp_space].a & AV_REMOVE) != 0)
          {
-            reindent_line(next, pc->column + pc->len);
+            reindent_line(next, pc->column + pc->len());
          }
       }
 
@@ -1795,6 +1783,6 @@ void indent_preproc(void)
       }
 
       LOG_FMT(LPPIS, "%s: Indent line %d to %d (len %d, next->col %d)\n",
-              __func__, pc->orig_line, 1 + pp_level, pc->len, next->column);
+              __func__, pc->orig_line, 1 + pp_level, pc->len(), next->column);
    }
 }
