@@ -11,16 +11,13 @@
 
 #include "base_types.h"
 #include <cstring>     /* memset() */
-
+#include <bitset>
 
 typedef UINT8   log_sev_t;
 
 
 /** A simple array of 256 bits */
-typedef struct
-{
-   UINT8 bits[32];   /* 256 levels */
-} log_mask_t;
+typedef std::bitset<256> log_mask_t;
 
 
 /**
@@ -29,9 +26,9 @@ typedef struct
  * @param sev  The severity to check
  * @return     true (is set) or false (not set)
  */
-static_inline bool logmask_test(const log_mask_t *mask, log_sev_t sev)
+static_inline bool logmask_test(const log_mask_t& mask, log_sev_t sev)
 {
-   return((mask->bits[sev >> 3] & (1 << (sev & 0x07))) != 0);
+   return(mask.test(sev));
 }
 
 
@@ -41,16 +38,9 @@ static_inline bool logmask_test(const log_mask_t *mask, log_sev_t sev)
  * @param sev     The severity to check
  * @param value   true (set bit) or false (clear bit)
  */
-static_inline void logmask_set_sev(log_mask_t *mask, log_sev_t sev, bool value)
+static_inline void logmask_set_sev(log_mask_t& mask, log_sev_t sev, bool value)
 {
-   if (value)
-   {
-      mask->bits[sev >> 3] |= (1 << (sev & 0x07));
-   }
-   else
-   {
-      mask->bits[sev >> 3] &= ~(1 << (sev & 0x07));
-   }
+   mask.set(sev, value);
 }
 
 
@@ -59,9 +49,16 @@ static_inline void logmask_set_sev(log_mask_t *mask, log_sev_t sev, bool value)
  *
  * @param value   true (set bit) or false (clear bit)
  */
-static_inline void logmask_set_all(log_mask_t *mask, bool value)
+static_inline void logmask_set_all(log_mask_t& mask, bool value)
 {
-   memset(mask->bits, value ? 0xff : 0, sizeof(mask->bits));
+   if (value)
+   {
+      mask.set();
+   }
+   else
+   {
+      mask.reset();
+   }
 }
 
 
@@ -75,7 +72,7 @@ static_inline void logmask_set_all(log_mask_t *mask, bool value)
  * @param size the size of the buffer
  * @return     buf (pass through)
  */
-char *logmask_to_str(const log_mask_t *mask, char *buf, int size);
+char *logmask_to_str(const log_mask_t& mask, char *buf, int size);
 
 
 /**
@@ -84,7 +81,7 @@ char *logmask_to_str(const log_mask_t *mask, char *buf, int size);
  * @param str     The string to parse
  * @param mask    The mask to populate
  */
-void logmask_from_string(const char *str, log_mask_t *mask);
+void logmask_from_string(const char *str, log_mask_t& mask);
 
 
-#endif   /* LOGMASK_H_INCLUDED */
+#endif /* LOGMASK_H_INCLUDED */
