@@ -4,8 +4,6 @@
  *
  * @author  Ben Gardner
  * @license GPL v2+
- *
- * $Id$
  */
 #ifndef MD5_H_INCLUDED
 #define MD5_H_INCLUDED
@@ -16,10 +14,11 @@ class MD5
 {
 public:
    MD5();
-   ~MD5() { };
+   ~MD5() { }
 
    void Init();
    void Update(const void *data, UINT32 len);
+
    void Final(UINT8 digest[16]);
 
    /* internal function */
