@@ -9,7 +9,6 @@
 #include "prototypes.h"
 #include "chunk_list.h"
 #include "unc_ctype.h"
-#include <cstring>
 #include <cstdlib>
 
 static void output_comment_multi(chunk_t *pc);
@@ -24,27 +23,27 @@ struct cmt_reflow
    int        word_count;  /* number of words on this line */
    bool       kw_subst;    /* do keyword substitution */
    int        xtra_indent; /* extra indent of non-first lines (0 or 1) */
-   const char *cont_text;  /* fixed text to output at the start of a line (0 to 3 chars) */
+   unc_text   cont_text;   /* fixed text to output at the start of a line (0 to 3 chars) */
    bool       reflow;      /* reflow the current line */
 };
 
 
 static chunk_t *output_comment_c(chunk_t *pc);
 static chunk_t *output_comment_cpp(chunk_t *pc);
-static void add_comment_text(const char *text, int len,
+static void add_comment_text(const unc_text& text,
                              cmt_reflow& cmt, bool esc_close);
 
 /**
  * All output text is sent here, one char at a time.
  */
-static void add_char(char ch)
+static void add_char(UINT32 ch)
 {
-   static char last_char = 0;
+   static int last_char = 0;
 
    /* If we did a '\r' and it isn't followed by a '\n', then output a newline */
    if ((last_char == '\r') && (ch != '\n'))
    {
-      fputs(cpd.newline, cpd.fout);
+      write_string(cpd.fout, cpd.newline.get(), cpd.enc);
       cpd.column      = 1;
       cpd.did_newline = 1;
       cpd.spaces      = 0;
@@ -53,7 +52,7 @@ static void add_char(char ch)
    /* convert a newline into the LF/CRLF/CR sequence */
    if (ch == '\n')
    {
-      fputs(cpd.newline, cpd.fout);
+      write_string(cpd.fout, cpd.newline.get(), cpd.enc);
       cpd.column      = 1;
       cpd.did_newline = 1;
       cpd.spaces      = 0;
@@ -86,10 +85,10 @@ static void add_char(char ch)
       {
          while (cpd.spaces > 0)
          {
-            fputc(' ', cpd.fout);
+            write_char(cpd.fout, ' ', cpd.enc);
             cpd.spaces--;
          }
-         fputc(ch, cpd.fout);
+         write_char(cpd.fout, ch, cpd.enc);
          if (ch == '\t')
          {
             cpd.column = next_tab_column(cpd.column);
@@ -104,24 +103,23 @@ static void add_char(char ch)
 }
 
 
-static void add_text(const char *text)
+static void add_text(const char *ascii_text)
 {
    char ch;
 
-   while ((ch = *text) != 0)
+   while ((ch = *ascii_text) != 0)
    {
-      text++;
+      ascii_text++;
       add_char(ch);
    }
 }
 
 
-static void add_text_len(const char *text, int len)
+static void add_text(const unc_text& text)
 {
-   while (len-- > 0)
+   for (int idx = 0; idx < text.size(); idx++)
    {
-      add_char(*text);
-      text++;
+      add_char(text[idx]);
    }
 }
 
@@ -130,21 +128,21 @@ static void add_text_len(const char *text, int len)
  * Count the number of characters to the end of the next chunk of text.
  * If it exceeds the limit, return true.
  */
-static bool next_word_exceeds_limit(const char *text)
+static bool next_word_exceeds_limit(const unc_text& text, int idx)
 {
    int length = 0;
 
    /* Count any whitespace */
-   while ((*text != 0) && unc_isspace(*text))
+   while ((idx < text.size()) && unc_isspace(text[idx]))
    {
-      text++;
+      idx++;
       length++;
    }
 
    /* Count non-whitespace */
-   while ((*text != 0) && !unc_isspace(*text))
+   while ((idx < text.size()) && !unc_isspace(text[idx]))
    {
-      text++;
+      idx++;
       length++;
    }
    return((cpd.column + length - 1) > cpd.settings[UO_cmt_width].n);
@@ -240,7 +238,7 @@ void output_parsed(FILE *pfile)
               pc->brace_level, pc->level, pc->pp_level,
               pc->flags, pc->nl_count, pc->after_tab);
 
-      if ((pc->type != CT_NEWLINE) && (pc->len != 0))
+      if ((pc->type != CT_NEWLINE) && (pc->len() != 0))
       {
          for (cnt = 0; cnt < pc->column; cnt++)
          {
@@ -248,7 +246,7 @@ void output_parsed(FILE *pfile)
          }
          if (pc->type != CT_NL_CONT)
          {
-            fprintf(pfile, "%.*s", pc->len, pc->str);
+            fprintf(pfile, "%s", pc->str.c_str());
          }
          else
          {
@@ -305,9 +303,21 @@ void output_text(FILE *pfile)
    cpd.did_newline = 1;
    cpd.column      = 1;
 
-   if (cpd.bom != NULL)
+   if (cpd.bom)
    {
-      add_text_len(cpd.bom->str, cpd.bom->len);
+      write_bom(pfile, cpd.enc);
+   }
+
+   if (cpd.frag_cols > 0)
+   {
+      int indent = cpd.frag_cols - 1;
+
+      for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next(pc))
+      {
+         pc->column        += indent;
+         pc->column_indent += indent;
+      }
+      cpd.frag_cols = 0;
    }
 
    for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next(pc))
@@ -378,12 +388,12 @@ void output_text(FILE *pfile)
       {
          pc = output_comment_c(pc);
       }
-      else if (pc->type == CT_JUNK)
+      else if ((pc->type == CT_JUNK) || (pc->type == CT_IGNORED))
       {
          /* do not adjust the column for junk */
-         add_text_len(pc->str, pc->len);
+         add_text(pc->str);
       }
-      else if (pc->len == 0)
+      else if (pc->len() == 0)
       {
          /* don't do anything for non-visible stuff */
          LOG_FMT(LOUTIND, " <%d> -", pc->column);
@@ -437,22 +447,19 @@ void output_text(FILE *pfile)
             }
 
             /* not the first item on a line */
+            prev       = chunk_get_prev(pc);
+            allow_tabs = (cpd.settings[UO_align_with_tabs].b &&
+                          ((pc->flags & PCF_WAS_ALIGNED) != 0) &&
+                          ((prev->column + prev->len() + 1) != pc->column));
             if (cpd.settings[UO_align_keep_tabs].b)
             {
-               allow_tabs = pc->after_tab;
-            }
-            else
-            {
-               prev       = chunk_get_prev(pc);
-               allow_tabs = (cpd.settings[UO_align_with_tabs].b &&
-                             ((pc->flags & PCF_WAS_ALIGNED) != 0) &&
-                             ((prev->column + prev->len + 1) != pc->column));
+               allow_tabs |= pc->after_tab;
             }
             LOG_FMT(LOUTIND, " %d(%d) -", pc->column, allow_tabs);
          }
 
          output_to_column(pc->column, allow_tabs);
-         add_text_len(pc->str, pc->len);
+         add_text(pc->str);
          cpd.did_newline = chunk_is_newline(pc);
       }
    }
@@ -464,21 +471,21 @@ void output_text(FILE *pfile)
  * @param line the comment line
  * @return 0=not present, >0=number of chars that are part of the lead
  */
-static int cmt_parse_lead(const char *line, int is_last)
+static int cmt_parse_lead(const unc_text& line, int is_last)
 {
    int len = 0;
 
-   while ((len < 32) && (line[len] != 0))
+   while ((len < 32) && (len < line.size()))
    {
       if ((len > 0) && (line[len] == '/'))
       {
          /* ignore combined comments */
          int tmp = len + 1;
-         while (unc_isspace(line[tmp]))
+         while ((tmp < line.size()) && unc_isspace(line[tmp]))
          {
             tmp++;
          }
-         if (line[tmp] == '/')
+         if ((tmp < line.size()) && (line[tmp] == '/'))
          {
             return 1;
          }
@@ -496,7 +503,7 @@ static int cmt_parse_lead(const char *line, int is_last)
       return 1;
    }
 
-   if ((len > 0) && ((line[len] == 0) || unc_isspace(line[len])))
+   if ((len > 0) && ((len >= line.size()) || unc_isspace(line[len])))
    {
       return len;
    }
@@ -532,12 +539,13 @@ static int cmt_parse_lead(const char *line, int is_last)
  * @param start_col Starting column
  * @return 0 or 1
  */
-static void calculate_comment_body_indent(cmt_reflow &cmt, const char *str, int len)
+static void calculate_comment_body_indent(cmt_reflow &cmt, const unc_text& str)
 {
    int idx       = 0;
    int first_len = 0;
    int last_len  = 0;
    int width     = 0;
+   int len       = str.size();
 
    cmt.xtra_indent = 0;
 
@@ -710,14 +718,14 @@ static void add_comment_javaparam(chunk_t *pc, cmt_reflow& cmt)
          {
             if (need_nl)
             {
-               add_comment_text("\n ", 2, cmt, false);
+               add_comment_text("\n ", cmt, false);
             }
             need_nl = true;
             add_text("@param");
             if (prev != NULL)
             {
                add_text(" ");
-               add_text_len(prev->str, prev->len);
+               add_text(prev->str);
                add_text(" TODO");
             }
             prev = NULL;
@@ -739,7 +747,7 @@ static void add_comment_javaparam(chunk_t *pc, cmt_reflow& cmt)
    {
       if (need_nl)
       {
-         add_comment_text("\n ", 2, cmt, false);
+         add_comment_text("\n ", cmt, false);
       }
       add_text("@return TODO");
    }
@@ -751,19 +759,19 @@ static void add_comment_javaparam(chunk_t *pc, cmt_reflow& cmt)
  * on that keyword.
  * @return the number of characters eaten from the text
  */
-static int add_comment_kw(const char *text, int len, cmt_reflow& cmt)
+static int add_comment_kw(const unc_text& text, int idx, cmt_reflow& cmt)
 {
-   if ((len >= 11) && (memcmp(text, "$(filename)", 11) == 0))
+   if (text.startswith("$(filename)", idx))
    {
       add_text(path_basename(cpd.filename));
       return(11);
    }
-   if ((len >= 8) && (memcmp(text, "$(class)", 8) == 0))
+   if (text.startswith("$(class)", idx))
    {
       chunk_t *tmp = get_next_class(cmt.pc);
       if (tmp != NULL)
       {
-         add_text_len(tmp->str, tmp->len);
+         add_text(tmp->str);
          return(8);
       }
    }
@@ -775,21 +783,49 @@ static int add_comment_kw(const char *text, int len, cmt_reflow& cmt)
       return(0);
    }
 
-   if ((len >= 11) && (memcmp(text, "$(function)", 11) == 0))
+   if (text.startswith("$(message)", idx))
+   {
+      add_text(fcn->str);
+      chunk_t *tmp = chunk_get_next_ncnl(fcn);
+      chunk_t *word = NULL;
+      while (tmp)
+      {
+         if ((tmp->type == CT_BRACE_OPEN) || (tmp->type == CT_SEMICOLON))
+         {
+            break;
+         }
+         if (tmp->type == CT_OC_COLON)
+         {
+            if (word != NULL)
+            {
+               add_text(word->str);
+               word = NULL;
+            }
+            add_text(":");
+         }
+         if (tmp->type == CT_WORD)
+         {
+            word = tmp;
+         }
+         tmp = chunk_get_next_ncnl(tmp);
+      }
+      return(10);
+   }
+   if (text.startswith("$(function)", idx))
    {
       if (fcn->parent_type == CT_OPERATOR)
       {
-         add_text_len("operator ", 9);
+         add_text("operator ");
       }
-      add_text_len(fcn->str, fcn->len);
+      add_text(fcn->str);
       return(11);
    }
-   if ((len >= 12) && (memcmp(text, "$(javaparam)", 12) == 0))
+   if (text.startswith("$(javaparam)", idx))
    {
       add_comment_javaparam(fcn, cmt);
       return(12);
    }
-   if ((len >= 9) && (memcmp(text, "$(fclass)", 9) == 0))
+   if (text.startswith("$(fclass)", idx))
    {
       chunk_t *tmp = chunk_get_prev_ncnl(fcn);
       if ((tmp != NULL) && (tmp->type == CT_OPERATOR))
@@ -800,7 +836,7 @@ static int add_comment_kw(const char *text, int len, cmt_reflow& cmt)
                             (tmp->type == CT_MEMBER)))
       {
          tmp = chunk_get_prev_ncnl(tmp);
-         add_text_len(tmp->str, tmp->len);
+         add_text(tmp->str);
          return(9);
       }
    }
@@ -808,6 +844,24 @@ static int add_comment_kw(const char *text, int len, cmt_reflow& cmt)
 }
 
 
+static int next_up(const unc_text& text, int idx, unc_text& tag)
+{
+   int offs = 0;
+
+   while ((idx < text.size()) && unc_isspace(text[idx]))
+   {
+      idx++;
+      offs++;
+   }
+
+   if (text.startswith(tag, idx))
+   {
+      return(offs);
+   }
+   return(-1);
+}
+
+
 /**
  * Outputs a comment. The initial opening '//' may be included in the text.
  * Subsequent openings (if combining comments), should not be included.
@@ -820,20 +874,22 @@ static int add_comment_kw(const char *text, int len, cmt_reflow& cmt)
  * If the last char on a line is a ':' or '.', then the next line won't be
  * combined.
  */
-static void add_comment_text(const char *text, int len,
+static void add_comment_text(const unc_text& text,
                              cmt_reflow& cmt, bool esc_close)
 {
    bool was_star   = false;
    bool was_slash  = false;
    bool was_dollar = false;
    bool in_word    = false;
+   int  tmp;
+   int  len = text.size();
 
    for (int idx = 0; idx < len; idx++)
    {
       if (!was_dollar && cmt.kw_subst &&
           (text[idx] == '$') && (len > (idx + 3)) && (text[idx + 1] == '('))
       {
-         idx += add_comment_kw(&text[idx], len - idx, cmt);
+         idx += add_comment_kw(text, idx, cmt);
          if (idx >= len)
          {
             break;
@@ -850,13 +906,23 @@ static void add_comment_text(const char *text, int len,
          {
             add_char(' ');
          }
-         add_text(cmt.cont_text);
+
+         /* hack to get escaped newlines to align and not dup the leading '//' */
+         tmp = next_up(text, idx + 1, cmt.cont_text);
+         if (tmp < 0)
+         {
+            add_text(cmt.cont_text);
+         }
+         else
+         {
+            idx += tmp;
+         }
       }
       else if (cmt.reflow &&
                (text[idx] == ' ') &&
                (cpd.settings[UO_cmt_width].n > 0) &&
                ((cpd.column > cpd.settings[UO_cmt_width].n) ||
-                next_word_exceeds_limit(text + idx)))
+                next_word_exceeds_limit(text, idx)))
       {
          in_word = false;
          add_char('\n');
@@ -881,6 +947,7 @@ static void add_comment_text(const char *text, int len,
             cmt.word_count++;
          }
          in_word = !unc_isspace(text[idx]);
+
          add_char(text[idx]);
          was_star   = (text[idx] == '*');
          was_slash  = (text[idx] == '/');
@@ -899,7 +966,7 @@ static void output_cmt_start(cmt_reflow& cmt, chunk_t *pc)
    cmt.word_count  = 0;
    cmt.kw_subst    = false;
    cmt.xtra_indent = 0;
-   cmt.cont_text   = "";
+   cmt.cont_text.clear();
    cmt.reflow      = false;
 
    if (cmt.brace_col == 0)
@@ -929,7 +996,7 @@ static void output_cmt_start(cmt_reflow& cmt, chunk_t *pc)
       chunk_t *prev = chunk_get_prev(pc);
       if (prev != NULL)
       {
-         int col_min = prev->column + prev->len + 1;
+         int col_min = prev->column + prev->len() + 1;
          if (cmt.column < col_min)
          {
             cmt.column = col_min;
@@ -942,7 +1009,7 @@ static void output_cmt_start(cmt_reflow& cmt, chunk_t *pc)
        ((pc->parent_type == CT_COMMENT_END) ||
         (pc->parent_type == CT_COMMENT_WHOLE)))
    {
-      cmt.column = next_tab_column(cmt.column - 1);
+      cmt.column = align_tab_column(cmt.column - 1);
       //LOG_FMT(LSYS, "%s: line %d, orig:%d new:%d\n",
       //        __func__, pc->orig_line, pc->column, cmt.column);
       pc->column = cmt.column;
@@ -1013,29 +1080,32 @@ static chunk_t *output_comment_c(chunk_t *first)
        !can_combine_comment(first, cmt))
    {
       /* Just add the single comment */
-      add_comment_text(first->str, first->len, cmt, false);
+      add_comment_text(first->str, cmt, false);
       return(first);
    }
 
-   add_text_len("/*", 2);
+   add_text("/*");
    if (cpd.settings[UO_cmt_c_nl_start].b)
    {
-      add_comment_text("\n", 1, cmt, false);
+      add_comment_text("\n", cmt, false);
    }
-   chunk_t *pc = first;
+   chunk_t  *pc = first;
+   unc_text tmp;
    while (can_combine_comment(pc, cmt))
    {
-      add_comment_text(pc->str + 2, pc->len - 4, cmt, false);
-      add_comment_text("\n", 1, cmt, false);
+      tmp.set(pc->str, 2, pc->len() - 4);
+      add_comment_text(tmp, cmt, false);
+      add_comment_text("\n", cmt, false);
       pc = chunk_get_next(chunk_get_next(pc));
    }
-   add_comment_text(pc->str + 2, pc->len - 4, cmt, false);
+   tmp.set(pc->str, 2, pc->len() - 4);
+   add_comment_text(tmp, cmt, false);
    if (cpd.settings[UO_cmt_c_nl_end].b)
    {
       cmt.cont_text = " ";
-      add_comment_text("\n", 1, cmt, false);
+      add_comment_text("\n", cmt, false);
    }
-   add_comment_text("*/", 2, cmt, false);
+   add_comment_text("*/", cmt, false);
    return(pc);
 }
 
@@ -1049,6 +1119,7 @@ static chunk_t *output_comment_c(chunk_t *first)
 static chunk_t *output_comment_cpp(chunk_t *first)
 {
    cmt_reflow cmt;
+   unc_text   tmp;
 
    output_cmt_start(cmt, first);
    cmt.reflow = (cpd.settings[UO_cmt_reflow_mode].n != 1);
@@ -1060,33 +1131,32 @@ static chunk_t *output_comment_cpp(chunk_t *first)
 
       if (cpd.settings[UO_sp_cmt_cpp_start].a == AV_IGNORE)
       {
-         add_comment_text(first->str, first->len, cmt, false);
+         add_comment_text(first->str, cmt, false);
       }
       else
       {
-         add_comment_text(first->str, 2, cmt, false);
+         unc_text tmp(first->str, 0, 2);
+         add_comment_text(tmp, cmt, false);
 
-         const char *tmp = first->str + 2;
-         int        len  = first->len - 2;
+         tmp.set(first->str, 2, first->len() - 2);
 
          if (cpd.settings[UO_sp_cmt_cpp_start].a & AV_REMOVE)
          {
-            while ((len > 0) && unc_isspace(*tmp))
+            while ((tmp.size() > 0) && unc_isspace(tmp[0]))
             {
-               len--;
-               tmp++;
+               tmp.pop_front();
             }
          }
-         if (len > 0)
+         if (tmp.size() > 0)
          {
             if (cpd.settings[UO_sp_cmt_cpp_start].a & AV_ADD)
             {
-               if (!unc_isspace(*tmp))
+               if (!unc_isspace(tmp[0]))
                {
-                  add_comment_text(" ", 1, cmt, false);
+                  add_comment_text(" ", cmt, false);
                }
             }
-            add_comment_text(tmp, len, cmt, false);
+            add_comment_text(tmp, cmt, false);
          }
       }
 
@@ -1101,24 +1171,25 @@ static chunk_t *output_comment_cpp(chunk_t *first)
        !can_combine_comment(first, cmt))
    {
       /* nothing to group: just output a single line */
-      add_text_len("/*", 2);
+      add_text("/*");
       if (!unc_isspace(first->str[2]))
       {
          add_char(' ');
       }
-      add_comment_text(first->str + 2, first->len - 2, cmt, true);
-      add_text_len(" */", 3);
+      tmp.set(first->str, 2, first->len() - 2);
+      add_comment_text(tmp, cmt, true);
+      add_text(" */");
       return(first);
    }
 
-   add_text_len("/*", 2);
+   add_text("/*");
    if (cpd.settings[UO_cmt_cpp_nl_start].b)
    {
-      add_comment_text("\n", 1, cmt, false);
+      add_comment_text("\n", cmt, false);
    }
    else
    {
-      add_text_len(" ", 1);
+      add_text(" ");
    }
    chunk_t *pc = first;
 
@@ -1126,51 +1197,53 @@ static chunk_t *output_comment_cpp(chunk_t *first)
    while (can_combine_comment(pc, cmt))
    {
       offs = unc_isspace(pc->str[2]) ? 1 : 0;
-      add_comment_text(pc->str + 2 + offs, pc->len - (2 + offs), cmt, true);
-      add_comment_text("\n", 1, cmt, false);
+      tmp.set(pc->str, 2 + offs, pc->len() - (2 + offs));
+      add_comment_text(tmp, cmt, true);
+      add_comment_text("\n", cmt, false);
       pc = chunk_get_next(chunk_get_next(pc));
    }
    offs = unc_isspace(pc->str[2]) ? 1 : 0;
-   add_comment_text(pc->str + 2 + offs, pc->len - (2 + offs), cmt, true);
+   tmp.set(pc->str, 2 + offs, pc->len() - (2 + offs));
+   add_comment_text(tmp, cmt, true);
    if (cpd.settings[UO_cmt_cpp_nl_end].b)
    {
       cmt.cont_text = "";
-      add_comment_text("\n", 1, cmt, false);
+      add_comment_text("\n", cmt, false);
    }
-   add_comment_text(" */", 3, cmt, false);
+   add_comment_text(" */", cmt, false);
    return(pc);
 }
 
 
-static void cmt_trim_whitespace(int& line_len, char *line, bool in_preproc)
+static void cmt_trim_whitespace(unc_text& line, bool in_preproc)
 {
    /* Remove trailing whitespace on the line */
-   while ((line_len > 0) &&
-          ((line[line_len - 1] == ' ') ||
-           (line[line_len - 1] == '\t')))
+   while ((line.size() > 0) &&
+          ((line.back() == ' ') ||
+           (line.back() == '\t')))
    {
-      line_len--;
+      line.pop_back();
    }
 
    /* If in a preproc, shift any bs-nl back to the comment text */
-   if (in_preproc && (line_len > 1) && (line[line_len - 1] == '\\'))
+   if (in_preproc && (line.size() > 1) && (line.back() == '\\'))
    {
       bool do_space = false;
 
       /* If there was any space before the backslash, change it to 1 space */
-      line_len--;
-      while ((line_len > 0) &&
-             ((line[line_len - 1] == ' ') ||
-              (line[line_len - 1] == '\t')))
+      line.pop_back();
+      while ((line.size() > 0) &&
+             ((line.back() == ' ') ||
+              (line.back() == '\t')))
       {
          do_space = true;
-         line_len--;
+         line.pop_back();
       }
       if (do_space)
       {
-         line[line_len++] = ' ';
+         line.append(' ');
       }
-      line[line_len++] = '\\';
+      line.append('\\');
    }
 }
 
@@ -1183,16 +1256,13 @@ static void cmt_trim_whitespace(int& line_len, char *line, bool in_preproc)
 static void output_comment_multi(chunk_t *pc)
 {
    int        cmt_col;
-   const char *cmt_str;
-   int        remaining;
-   char       ch;
+   int        cmt_idx;
+   int        ch;
    chunk_t    *prev;
-   char       *line;
-   int        line_len;
+   unc_text   line;
    int        line_count = 0;
    int        ccol; /* the col of subsequent comment lines */
    int        col_diff = 0;
-   char       lead[80];
    bool       nl_end = false;
    cmt_reflow cmt;
 
@@ -1205,38 +1275,33 @@ static void output_comment_multi(chunk_t *pc)
    cmt_col = cmt.base_col;
    col_diff = pc->orig_col - cmt.base_col;
 
-   calculate_comment_body_indent(cmt, pc->str, pc->len);
+   calculate_comment_body_indent(cmt, pc->str);
 
    cmt.cont_text = !cpd.settings[UO_cmt_indent_multi].b ? "" :
                    (cpd.settings[UO_cmt_star_cont].b ? "* " : "  ");
 
    //LOG_FMT(LSYS, "Indenting1 line %d to col %d (orig=%d) col_diff=%d xtra=%d cont='%s'\n",
-   //        pc->orig_line, cmt_col, pc->orig_col, col_diff, cmt.xtra_indent, cmt.cont_text);
+   //        pc->orig_line, cmt_col, pc->orig_col, col_diff, cmt.xtra_indent, cmt.cont_text.c_str());
 
-   ccol      = pc->column;
-   remaining = pc->len;
-   cmt_str   = pc->str;
-   line_len  = 0;
-   line      = new char[remaining + 1024 + 1]; /* + 1 for '\0' */
-   while (remaining > 0)
+   ccol    = pc->column;
+   cmt_idx = 0;
+   line.clear();
+   while (cmt_idx < pc->len())
    {
-      ch = *cmt_str;
-      cmt_str++;
-      remaining--;
+      ch = pc->str[cmt_idx++];
 
       /* handle the CRLF and CR endings. convert both to LF */
       if (ch == '\r')
       {
          ch = '\n';
-         if (*cmt_str == '\n')
+         if ((cmt_idx < pc->len()) && (pc->str[cmt_idx] == '\n'))
          {
-            cmt_str++;
-            remaining--;
+            cmt_idx++;
          }
       }
 
       /* Find the start column */
-      if (line_len == 0)
+      if (line.size() == 0)
       {
          nl_end = false;
          if (ch == ' ')
@@ -1261,16 +1326,15 @@ static void output_comment_multi(chunk_t *pc)
        */
       if ((cpd.settings[UO_cmt_reflow_mode].n == 2) &&
           (ch == '\n') &&
-          (remaining > 0))
+          (cmt_idx < pc->len()))
       {
          int  nxt_len            = 0;
          int  next_nonempty_line = -1;
          int  prev_nonempty_line = -1;
-         int  nwidx          = line_len;
+         int  nwidx          = line.size();
          bool star_is_bullet = false;
 
          /* strip trailing whitespace from the line collected so far */
-         line[nwidx] = 0; // sentinel
          while (nwidx > 0)
          {
             nwidx--;
@@ -1287,20 +1351,21 @@ static void output_comment_multi(chunk_t *pc)
             }
          }
 
+         int remaining = pc->len() - cmt_idx;
          for (nxt_len = 0;
               (nxt_len <= remaining) &&
-              (cmt_str[nxt_len] != 'r') &&
-              (cmt_str[nxt_len] != '\n');
+              (pc->str[nxt_len] != 'r') &&
+              (pc->str[nxt_len] != '\n');
               nxt_len++)
          {
             if ((next_nonempty_line < 0) &&
-                !unc_isspace(cmt_str[nxt_len]) &&
-                (cmt_str[nxt_len] != '*') &&
+                !unc_isspace(pc->str[nxt_len]) &&
+                (pc->str[nxt_len] != '*') &&
                 ((nxt_len == remaining) ||
                  ((pc->flags & PCF_IN_PREPROC)
-                  ? (cmt_str[nxt_len] != '\\') ||
-                  ((cmt_str[nxt_len + 1] != 'r') &&
-                   (cmt_str[nxt_len + 1] != '\n'))
+                  ? (pc->str[nxt_len] != '\\') ||
+                  ((pc->str[nxt_len + 1] != 'r') &&
+                   (pc->str[nxt_len + 1] != '\n'))
                   : true)))
             {
                next_nonempty_line = nxt_len; // first nonwhitespace char in the next line
@@ -1340,26 +1405,25 @@ static void output_comment_multi(chunk_t *pc)
          if ((prev_nonempty_line >= 0) && (next_nonempty_line >= 0) &&
              (((unc_isalnum(line[prev_nonempty_line]) ||
                 strchr(",)]", line[prev_nonempty_line])) &&
-               (unc_isalnum(cmt_str[next_nonempty_line]) ||
-                strchr("([", cmt_str[next_nonempty_line]))) ||
+               (unc_isalnum(pc->str[next_nonempty_line]) ||
+                strchr("([", pc->str[next_nonempty_line]))) ||
               (('.' == line[prev_nonempty_line]) &&    // dot followed by non-capital is NOT a new sentence start
-               unc_isupper(cmt_str[next_nonempty_line]))) &&
+               unc_isupper(pc->str[next_nonempty_line]))) &&
              !star_is_bullet)
          {
             // rewind the line to the last non-alpha:
-            line_len = prev_nonempty_line + 1;
+            line.resize(prev_nonempty_line + 1);
             // roll the current line forward to the first non-alpha:
-            cmt_str   += next_nonempty_line;
-            remaining -= next_nonempty_line;
+            cmt_idx += next_nonempty_line;
             // override the NL and make it a single whitespace:
             ch = ' ';
          }
       }
 
-      line[line_len++] = ch;
+      line.append(ch);
 
       /* If we just hit an end of line OR we just hit end-of-comment... */
-      if ((ch == '\n') || (remaining == 0))
+      if ((ch == '\n') || (cmt_idx == pc->len()))
       {
          line_count++;
 
@@ -1367,17 +1431,16 @@ static void output_comment_multi(chunk_t *pc)
          if (ch == '\n')
          {
             nl_end = true;
-            line_len--;
-            cmt_trim_whitespace(line_len, line, pc->flags & PCF_IN_PREPROC);
+            line.pop_back();
+            cmt_trim_whitespace(line, pc->flags & PCF_IN_PREPROC);
          }
-         line[line_len] = 0;
 
          //LOG_FMT(LSYS, "[%3d]%s\n", ccol, line);
 
          if (line_count == 1)
          {
             /* this is the first line - add unchanged */
-            add_comment_text(line, line_len, cmt, false);
+            add_comment_text(line, cmt, false);
             if (nl_end)
             {
                add_char('\n');
@@ -1394,7 +1457,7 @@ static void output_comment_multi(chunk_t *pc)
                ccol = cmt_col + 3;
             }
 
-            if (line_len == 0)
+            if (line.size() == 0)
             {
                /* Empty line - just a '\n' */
                if (cpd.settings[UO_cmt_star_cont].b)
@@ -1449,12 +1512,10 @@ static void output_comment_multi(chunk_t *pc)
 
                   int idx;
 
-                  idx = cmt_parse_lead(line, (remaining == 0));
+                  idx = cmt_parse_lead(line, (cmt_idx == pc->len()));
                   if (idx > 0)
                   {
-                     memcpy(lead, line, idx);
-                     lead[idx] = 0;
-                     cmt.cont_text = lead;
+                     cmt.cont_text.set(line, 0, idx);
                   }
                   else
                   {
@@ -1462,18 +1523,17 @@ static void output_comment_multi(chunk_t *pc)
                   }
                }
 
-               add_comment_text(line, line_len, cmt, false);
+               add_comment_text(line, cmt, false);
                if (nl_end)
                {
-                  add_text_len("\n", 1);
+                  add_text("\n");
                }
             }
          }
-         line_len = 0;
-         ccol     = 1;
+         line.clear();
+         ccol = 1;
       }
    }
-   delete[] line;
 }
 
 
@@ -1485,16 +1545,14 @@ static void output_comment_multi(chunk_t *pc)
 static void output_comment_multi_simple(chunk_t *pc)
 {
    int        cmt_col = pc->column;
-   const char *cmt_str;
-   int        remaining;
+   int        cmt_idx;
    char       ch;
-   char       line[1024];
-   int        line_len;
    int        line_count = 0;
    int        ccol;
    int        col_diff = 0;
    bool       nl_end   = false;
    cmt_reflow cmt;
+   unc_text   line;
 
    output_cmt_start(cmt, pc);
 
@@ -1511,29 +1569,25 @@ static void output_comment_multi_simple(chunk_t *pc)
       col_diff = 0;
    }
 
-   ccol      = pc->column;
-   remaining = pc->len;
-   cmt_str   = pc->str;
-   line_len  = 0;
-   while (remaining > 0)
+   ccol    = pc->column;
+   cmt_idx = 0;
+   line.clear();
+   while (cmt_idx < pc->len())
    {
-      ch = *cmt_str;
-      cmt_str++;
-      remaining--;
+      ch = pc->str[cmt_idx++];
 
       /* handle the CRLF and CR endings. convert both to LF */
       if (ch == '\r')
       {
          ch = '\n';
-         if (*cmt_str == '\n')
+         if ((cmt_idx < pc->len()) && (pc->str[cmt_idx] == '\n'))
          {
-            cmt_str++;
-            remaining--;
+            cmt_idx++;
          }
       }
 
       /* Find the start column */
-      if (line_len == 0)
+      if (line.size() == 0)
       {
          nl_end = false;
          if (ch == ' ')
@@ -1553,41 +1607,206 @@ static void output_comment_multi_simple(chunk_t *pc)
          }
       }
 
-      line[line_len++] = ch;
+      line.append(ch);
 
       /* If we just hit an end of line OR we just hit end-of-comment... */
-      if ((ch == '\n') || (remaining == 0))
+      if ((ch == '\n') || (cmt_idx == pc->len()))
       {
          line_count++;
 
          /* strip trailing tabs and spaces before the newline */
          if (ch == '\n')
          {
-            line_len--;
+            line.pop_back();
             nl_end = true;
 
             /* Say we aren't in a preproc to prevent changing any bs-nl */
-            cmt_trim_whitespace(line_len, line, false);
+            cmt_trim_whitespace(line, false);
          }
-         line[line_len] = 0;
 
          if (line_count > 1)
          {
             ccol -= col_diff;
          }
 
-         if (line_len > 0)
+         if (line.size() > 0)
          {
             cmt.column = ccol;
             cmt_output_indent(cmt.brace_col, cmt.base_col, cmt.column);
-            add_text_len(line, line_len);
+            add_text(line);
          }
          if (nl_end)
          {
             add_char('\n');
          }
-         line_len = 0;
-         ccol     = 1;
+         line.clear();
+         ccol = 1;
+      }
+   }
+}
+
+
+/**
+ * This renders the #if condition to a string buffer.
+ */
+static void generate_if_conditional_as_text(unc_text& dst, chunk_t *ifdef)
+{
+   chunk_t *pc;
+   int     column = -1;
+
+   dst.clear();
+   for (pc = ifdef; pc != NULL; pc = chunk_get_next(pc))
+   {
+      if (column == -1)
+      {
+         column = pc->column;
+      }
+      if ((pc->type == CT_NEWLINE) ||
+          (pc->type == CT_COMMENT_MULTI) ||
+          (pc->type == CT_COMMENT_CPP))
+      {
+         break;
+      }
+      else if (pc->type == CT_NL_CONT)
+      {
+         dst   += ' ';
+         column = -1;
+      }
+      else if ((pc->type == CT_COMMENT) ||
+               (pc->type == CT_COMMENT_EMBED))
+      {
+      }
+      else // if (pc->type == CT_JUNK) || else
+      {
+         int spacing;
+
+         for (spacing = pc->column - column; spacing > 0; spacing--)
+         {
+            dst += ' ';
+            column++;
+         }
+         dst.append(pc->str);
+         column += pc->len();
+      }
+   }
+}
+
+
+/*
+ * See also it's preprocessor counterpart
+ *   add_long_closebrace_comment
+ * in braces.cpp
+ *
+ * Note: since this concerns itself with the preprocessor -- which is line-oriented --
+ * it turns out that just looking at pc->pp_level is NOT the right thing to do.
+ * See a --parsed dump if you don't believe this: an '#endif' will be one level
+ * UP from the corresponding #ifdef when you look at the tokens 'ifdef' versus 'endif',
+ * but it's a whole another story when you look at their CT_PREPROC ('#') tokens!
+ *
+ * Hence we need to track and seek matching CT_PREPROC pp_levels here, which complicates
+ * things a little bit, but not much.
+ */
+void add_long_preprocessor_conditional_block_comment(void)
+{
+   chunk_t *pc;
+   chunk_t *tmp;
+   chunk_t *br_open;
+   chunk_t *br_close;
+   chunk_t *pp_start = NULL;
+   chunk_t *pp_end   = NULL;
+   int     nl_count;
+
+   for (pc = chunk_get_head(); pc; pc = chunk_get_next_ncnl(pc))
+   {
+      /* just track the preproc level: */
+      if (pc->type == CT_PREPROC)
+      {
+         pp_end = pp_start = pc;
+      }
+
+      if (pc->type != CT_PP_IF)
+      {
+         continue;
+      }
+#if 0
+      if ((pc->flags & PCF_IN_PREPROC) != 0)
+      {
+         continue;
+      }
+#endif
+
+      br_open  = pc;
+      nl_count = 0;
+
+      tmp = pc;
+      while ((tmp = chunk_get_next(tmp)) != NULL)
+      {
+         /* just track the preproc level: */
+         if (tmp->type == CT_PREPROC)
+         {
+            pp_end = tmp;
+         }
+
+         if (chunk_is_newline(tmp))
+         {
+            nl_count += tmp->nl_count;
+         }
+         else if ((pp_end->pp_level == pp_start->pp_level) &&
+                  ((tmp->type == CT_PP_ENDIF) ||
+                   (br_open->type == CT_PP_IF ? tmp->type == CT_PP_ELSE : 0)))
+         {
+            br_close = tmp;
+
+            LOG_FMT(LPPIF, "found #if / %s section on lines %d and %d, nl_count=%d\n",
+                    (tmp->type == CT_PP_ENDIF ? "#endif" : "#else"),
+                    br_open->orig_line, br_close->orig_line, nl_count);
+
+            /* Found the matching #else or #endif - make sure a newline is next */
+            tmp = chunk_get_next(tmp);
+
+            LOG_FMT(LPPIF, "next item type %d (is %s)\n",
+                    (tmp ? tmp->type : -1), (tmp ? chunk_is_newline(tmp) ? "newline"
+                                             : chunk_is_comment(tmp) ? "comment" : "other" : "---"));
+            if ((tmp == NULL) || (tmp->type == CT_NEWLINE) /* chunk_is_newline(tmp) */)
+            {
+               int nl_min;
+
+               if (br_close->type == CT_PP_ENDIF)
+               {
+                  nl_min = cpd.settings[UO_mod_add_long_ifdef_endif_comment].n;
+               }
+               else
+               {
+                  nl_min = cpd.settings[UO_mod_add_long_ifdef_else_comment].n;
+               }
+
+               const char *txt = !tmp ? "EOF" : ((tmp->type == CT_PP_ENDIF) ? "#endif" : "#else");
+               LOG_FMT(LPPIF, "#if / %s section candidate for augmenting when over NL threshold %d != 0 (nl_count=%d)\n",
+                       txt, nl_min, nl_count);
+
+               if ((nl_min > 0) && (nl_count > nl_min)) /* nl_count is 1 too large at all times as #if line was counted too */
+               {
+                  /* determine the added comment style */
+                  c_token_t style = (cpd.lang_flags & (LANG_CPP | LANG_CS)) ?
+                                    CT_COMMENT_CPP : CT_COMMENT;
+
+                  unc_text str;
+                  generate_if_conditional_as_text(str, br_open);
+
+                  LOG_FMT(LPPIF, "#if / %s section over threshold %d (nl_count=%d) --> insert comment after the %s: %s\n",
+                          txt, nl_min, nl_count, txt, str.c_str());
+
+                  /* Add a comment after the close brace */
+                  insert_comment_after(br_close, style, str);
+               }
+            }
+
+            /* checks both the #else and #endif for a given level, only then look further in the main loop */
+            if (br_close->type == CT_PP_ENDIF)
+            {
+               break;
+            }
+         }
       }
    }
 }
