@@ -59,10 +59,10 @@ static void add_parens_between(chunk_t *first, chunk_t *last)
    chunk_t *last_p;
    chunk_t *tmp;
 
-   LOG_FMT(LPARADD, "%s: line %d between %.*s [lvl=%d] and %.*s [lvl=%d]\n",
+   LOG_FMT(LPARADD, "%s: line %d between %s [lvl=%d] and %s [lvl=%d]\n",
            __func__, first->orig_line,
-           first->len, first->str, first->level,
-           last->len, last->str, last->level);
+           first->str.c_str(), first->level,
+           last->str.c_str(), last->level);
 
    /* Don't do anything if we have a bad sequence, ie "&& )" */
    first_n = chunk_get_next_ncnl(first);
@@ -71,11 +71,8 @@ static void add_parens_between(chunk_t *first, chunk_t *last)
       return;
    }
 
-   memset(&pc, 0, sizeof(pc));
-
    pc.type        = CT_PAREN_OPEN;
    pc.str         = "(";
-   pc.len         = 1;
    pc.flags       = first_n->flags & PCF_COPY_FLAGS;
    pc.level       = first_n->level;
    pc.pp_level    = first_n->pp_level;
@@ -137,13 +134,22 @@ static void check_bool_parens(chunk_t *popen, chunk_t *pclose, int nest)
    pc = popen;
    while (((pc = chunk_get_next_ncnl(pc)) != NULL) && (pc != pclose))
    {
+      if (pc->flags & PCF_IN_PREPROC)
+      {
+         LOG_FMT(LPARADD2, " -- bail on PP %s [%s] at line %d col %d, level %d\n",
+                 get_token_name(pc->type),
+                 pc->str.c_str(), pc->orig_line, pc->orig_col, pc->level);
+         return;
+      }
+
       if ((pc->type == CT_BOOL) ||
           (pc->type == CT_QUESTION) ||
-          (pc->type == CT_COND_COLON))
+          (pc->type == CT_COND_COLON) ||
+          (pc->type == CT_COMMA))
       {
-         LOG_FMT(LPARADD2, " -- %s [%.*s] at line %d col %d, level %d\n",
+         LOG_FMT(LPARADD2, " -- %s [%s] at line %d col %d, level %d\n",
                  get_token_name(pc->type),
-                 pc->len, pc->str, pc->orig_line, pc->orig_col, pc->level);
+                 pc->str.c_str(), pc->orig_line, pc->orig_col, pc->level);
          if (hit_compare)
          {
             hit_compare = false;
@@ -153,13 +159,13 @@ static void check_bool_parens(chunk_t *popen, chunk_t *pclose, int nest)
       }
       else if (pc->type == CT_COMPARE)
       {
-         LOG_FMT(LPARADD2, " -- compare [%.*s] at line %d col %d, level %d\n",
-                 pc->len, pc->str, pc->orig_line, pc->orig_col, pc->level);
+         LOG_FMT(LPARADD2, " -- compare [%s] at line %d col %d, level %d\n",
+                 pc->str.c_str(), pc->orig_line, pc->orig_col, pc->level);
          hit_compare = true;
       }
       else if (chunk_is_paren_open(pc))
       {
-         next = chunk_get_next_type(pc, (c_token_t)(pc->type + 1), pc->level);
+         next = chunk_skip_to_match(pc);
          if (next != NULL)
          {
             check_bool_parens(pc, next, nest + 1);
@@ -171,7 +177,7 @@ static void check_bool_parens(chunk_t *popen, chunk_t *pclose, int nest)
                (pc->type == CT_ANGLE_OPEN))
       {
          /* Skip [], {}, and <> */
-         pc = chunk_get_next_type(pc, (c_token_t)(pc->type + 1), pc->level);
+         pc = chunk_skip_to_match(pc);
       }
    }
 
