@@ -188,11 +188,21 @@ int pf_check(struct parse_frame *frm, chunk_t *pc)
    int        b4_cnt   = cpd.frame_count;
    int        pp_level = cpd.pp_level;
    const char *txt     = NULL;
+   chunk_t    *next;
 
    if (pc->type != CT_PREPROC)
    {
       return(pp_level);
    }
+   next = chunk_get_next(pc);
+
+   if (pc->parent_type != next->type)
+   {
+      LOG_FMT(LNOTE, "%s: Preproc parent not set correctly on line %d: got %s expected %s\n",
+              __func__, pc->orig_line, get_token_name(pc->parent_type),
+              get_token_name(next->type));
+      pc->parent_type = next->type;
+   }
 
    LOG_FMT(LPFCHK, "%s: %5d] %s\n",
            __func__, pc->orig_line, get_token_name(pc->parent_type));
