@@ -7,8 +7,6 @@
  *
  * @author  Ben Gardner
  * @license GPL v2+
- *
- * $Id: unc_ctype.h 1317 2008-08-04 01:06:49Z bengardner $
  */
 #ifndef UNC_CTYPE_H_INCLUDED
 #define UNC_CTYPE_H_INCLUDED
@@ -23,37 +21,59 @@ static inline int unc_fix_ctype(int ch)
    return((ch == -1) ? -1 : (ch & 0xff));
 }
 
+
 static inline int unc_isspace(int ch)
 {
    return(isspace(unc_fix_ctype(ch)));
 }
+
+
 static inline int unc_isprint(int ch)
 {
    return(isprint(unc_fix_ctype(ch)));
 }
+
+
 static inline int unc_isalpha(int ch)
 {
    return(isalpha(unc_fix_ctype(ch)));
 }
+
+
 static inline int unc_isalnum(int ch)
 {
    return(isalnum(unc_fix_ctype(ch)));
 }
+
+
 static inline int unc_toupper(int ch)
 {
    return(toupper(unc_fix_ctype(ch)));
 }
+
+
 static inline int unc_tolower(int ch)
 {
    return(tolower(unc_fix_ctype(ch)));
 }
+
+
 static inline int unc_isxdigit(int ch)
 {
    return(isxdigit(unc_fix_ctype(ch)));
 }
+
+
 static inline int unc_isdigit(int ch)
 {
    return(isdigit(unc_fix_ctype(ch)));
 }
 
+
+static inline int unc_isupper(int ch)
+{
+   return(isalpha(unc_fix_ctype(ch)) && (unc_toupper(unc_fix_ctype(ch)) == ch));
+}
+
+
 #endif /* UNC_CTYPE_H_INCLUDED */
