@@ -32,6 +32,8 @@
 #ifdef HAVE_STRINGS_H
 #include <strings.h>  /* strcasecmp() */
 #endif
+#include <vector>
+#include <deque>
 
 /* Global data */
 struct cp_data cpd;
@@ -40,10 +42,10 @@ struct cp_data cpd;
 static int language_from_tag(const char *tag);
 static int language_from_filename(const char *filename);
 static const char *language_to_string(int lang);
-static char *read_stdin(int& out_len);
-static void uncrustify_start(const char *data, int data_len);
+static bool read_stdin(file_mem& fm);
+static void uncrustify_start(const deque<int>& data);
 static void uncrustify_end();
-static void uncrustify_file(const char *data, int data_len, FILE *pfout,
+static void uncrustify_file(const file_mem& fm, FILE *pfout,
                             const char *parsed_file);
 static void do_source_file(const char *filename_in,
                            const char *filename_out,
@@ -156,6 +158,7 @@ static void usage_exit(const char *msg, const char *argv0, int code)
            " -l           : language override: C, CPP, D, CS, JAVA, PAWN, OC, OC+\n"
            " -t           : load a file with types (usually not needed)\n"
            " -q           : quiet mode - no output on stderr (-L will override)\n"
+           " --frag       : code fragment, assume the first line is indented correctly\n"
            "\n"
            "Config/Help Options:\n"
            " -h -? --help --usage     : print this message and exit\n"
@@ -223,8 +226,6 @@ static void redir_stdout(const char *output_file)
 
 int main(int argc, char *argv[])
 {
-   char       *data        = NULL;
-   int        data_len     = 0;
    const char *cfg_file    = "uncrustify.cfg";
    const char *parsed_file = NULL;
    const char *source_file = NULL;
@@ -271,15 +272,16 @@ int main(int argc, char *argv[])
    log_init(stderr);
    if (arg.Present("-q"))
    {
-      logmask_from_string("", &mask);
-      log_set_mask(&mask);
+      logmask_from_string("", mask);
+      log_set_mask(mask);
    }
    if (((p_arg = arg.Param("-L")) != NULL) ||
        ((p_arg = arg.Param("--log")) != NULL))
    {
-      logmask_from_string(p_arg, &mask);
-      log_set_mask(&mask);
+      logmask_from_string(p_arg, mask);
+      log_set_mask(mask);
    }
+   cpd.frag = arg.Present("--frag");
 
    if ((p_arg = arg.Param("--decode")) != NULL)
    {
@@ -345,7 +347,7 @@ int main(int argc, char *argv[])
    idx = 0;
    while ((p_arg = arg.Params("--type", idx)) != NULL)
    {
-      add_keyword(p_arg, CT_TYPE, LANG_ALL);
+      add_keyword(p_arg, CT_TYPE);
    }
 
    /* Load define files */
@@ -370,6 +372,10 @@ int main(int argc, char *argv[])
       {
          LOG_FMT(LWARN, "Ignoring unknown language: %s\n", p_arg);
       }
+      else
+      {
+         cpd.lang_forced = true;
+      }
    }
 
    /* Get the source file name */
@@ -470,7 +476,7 @@ int main(int argc, char *argv[])
       }
 
       /* Do some simple language detection based on the filename extension */
-      if (cpd.lang_flags == 0)
+      if (!cpd.lang_forced || (cpd.lang_flags == 0))
       {
          cpd.lang_flags = language_from_filename(source_file);
       }
@@ -483,10 +489,9 @@ int main(int argc, char *argv[])
          return(EXIT_FAILURE);
       }
 
-      uncrustify_start(fm.data, fm.length);
-      detect_options(fm.data, fm.length);
+      uncrustify_start(fm.data);
+      detect_options();
       uncrustify_end();
-      free(fm.data);
 
       redir_stdout(output_file);
       save_option_file(stdout, update_config_wd);
@@ -550,23 +555,21 @@ int main(int argc, char *argv[])
 
       redir_stdout(output_file);
 
-      data = read_stdin(data_len);
-      if (data == NULL)
+      file_mem fm;
+      if (!read_stdin(fm))
       {
-         LOG_FMT(LERR, "Out of memory\n");
+         LOG_FMT(LERR, "Failed to read stdin\n");
          return(100);
       }
 
       cpd.filename = "stdin";
 
       /* Done reading from stdin */
-      LOG_FMT(LSYS, "Parsing: %d bytes from stdin as language %s\n",
-              data_len, language_to_string(cpd.lang_flags));
+      LOG_FMT(LSYS, "Parsing: %d bytes (%d chars) from stdin as language %s\n",
+              (int)fm.raw.size(), (int)fm.data.size(),
+              language_to_string(cpd.lang_flags));
 
-      uncrustify_file(data, data_len, stdout, parsed_file);
-      free(data);
-      data_len = 0;
-      data     = NULL;
+      uncrustify_file(fm, stdout, parsed_file);
    }
    else if (source_file != NULL)
    {
@@ -652,57 +655,39 @@ static void process_source_list(const char *source_list,
 }
 
 
-static char *read_stdin(int& out_len)
+static bool read_stdin(file_mem& fm)
 {
-   char *data;
-   char *new_data;
-   int  data_size;
-   int  data_len;
-   int  len;
+   deque<UINT8> dq;
+   char         buf[4096];
+   int          len;
+   int          idx;
 
-   /* Start with 64k */
-   data_size = 64 * 1024;
-   data      = (char *)malloc(data_size);
-   data_len  = 0;
+   fm.raw.clear();
+   fm.data.clear();
+   fm.enc = ENC_ASCII;
 
-   if (data == NULL)
+   while (!feof(stdin))
    {
-      return(NULL);
-   }
-
-   while ((len = fread(&data[data_len], 1, data_size - data_len, stdin)) > 0)
-   {
-      data_len += len;
-      if (data_len == data_size)
+      len = fread(buf, 1, sizeof(buf), stdin);
+      for (idx = 0; idx < len; idx++)
       {
-         /* Double the buffer size */
-         data_size *= 2;
-         if ((new_data = (char *)realloc(data, data_size)) == NULL)
-         {
-            free(data);
-            return(NULL);
-         }
-         data = new_data;
+         dq.push_back(buf[idx]);
       }
    }
 
-   assert(data_len < data_size);
-   /* Make sure the buffer is terminated */
-   data[data_len] = 0;
-
-   out_len = data_len;
-
-   return(data);
+   /* Copy the raw data from the deque to the vector */
+   fm.raw.insert(fm.raw.end(), dq.begin(), dq.end());
+   return decode_unicode(fm.raw, fm.data, fm.enc, fm.bom);
 }
 
 
-static void make_folders(const char *filename)
+static void make_folders(const string& filename)
 {
    int  idx;
    int  last_idx = 0;
-   char outname[1024];
+   char outname[4096];
 
-   snprintf(outname, sizeof(outname), "%s", filename);
+   snprintf(outname, sizeof(outname), "%s", filename.c_str());
 
    for (idx = 0; outname[idx] != 0; idx++)
    {
@@ -741,8 +726,9 @@ static int load_mem_file(const char *filename, file_mem& fm)
    struct stat my_stat;
    FILE        *p_file;
 
-   fm.data   = NULL;
-   fm.length = 0;
+   fm.raw.clear();
+   fm.data.clear();
+   fm.enc = ENC_ASCII;
 
    /* Grab the stat info for the file */
    if (stat(filename, &my_stat) < 0)
@@ -762,32 +748,35 @@ static int load_mem_file(const char *filename, file_mem& fm)
       return(-1);
    }
 
-   fm.length = my_stat.st_size;
-   fm.data   = (char *)malloc(fm.length + 1);
-   if (fm.data == NULL)
+   fm.raw.resize(my_stat.st_size);
+   if (my_stat.st_size == 0)
    {
-      LOG_FMT(LERR, "%s: fopen(%s) failed: out of memory\n", __func__, filename);
-      cpd.error_count++;
-   }
-   else if (fread(fm.data, fm.length, 1, p_file) != 1)
-   {
-      LOG_FMT(LERR, "%s: fread(%s) failed: %s (%d)\n",
-              __func__, filename, strerror(errno), errno);
-      cpd.error_count++;
+      /* Empty file */
+      retval = 0;
+      fm.bom = false;
+      fm.enc = ENC_ASCII;
+      fm.data.clear();
    }
    else
    {
-      fm.data[fm.length] = 0;
-      retval             = 0;
+      /* read the raw data */
+      if (fread(&fm.raw[0], fm.raw.size(), 1, p_file) != 1)
+      {
+         LOG_FMT(LERR, "%s: fread(%s) failed: %s (%d)\n",
+                 __func__, filename, strerror(errno), errno);
+         cpd.error_count++;
+      }
+      else if (!decode_unicode(fm.raw, fm.data, fm.enc, fm.bom))
+      {
+         LOG_FMT(LERR, "%s: failed to decode the file '%s'\n", __func__, filename);
+      }
+      else
+      {
+         LOG_FMT(LNOTE, "%s: '%s' encoding looks like %d\n", __func__, filename, fm.enc);
+         retval = 0;
+      }
    }
    fclose(p_file);
-
-   if ((retval != 0) && (fm.data != NULL))
-   {
-      free(fm.data);
-      fm.data   = NULL;
-      fm.length = 0;
-   }
    return(retval);
 }
 
@@ -845,6 +834,12 @@ static int load_header_files()
       retval |= load_mem_file_config(cpd.settings[UO_cmt_insert_class_header].str,
                                      cpd.class_hdr);
    }
+   if ((cpd.settings[UO_cmt_insert_oc_msg_header].str != NULL) &&
+       (cpd.settings[UO_cmt_insert_oc_msg_header].str[0] != 0))
+   {
+      retval |= load_mem_file_config(cpd.settings[UO_cmt_insert_oc_msg_header].str,
+                                     cpd.oc_msg_hdr);
+   }
    return(retval);
 }
 
@@ -871,7 +866,7 @@ static const char *make_output_filename(char *buf, int buf_size,
 /**
  * Reinvent the wheel with a file comparision function...
  */
-static bool file_content_matches(const char *filename1, const char *filename2)
+static bool file_content_matches(const string& filename1, const string& filename2)
 {
    struct stat st1, st2;
    int         fd1, fd2;
@@ -880,18 +875,18 @@ static bool file_content_matches(const char *filename1, const char *filename2)
    int         minlen;
 
    /* Check the sizes first */
-   if ((stat(filename1, &st1) != 0) ||
-       (stat(filename2, &st2) != 0) ||
+   if ((stat(filename1.c_str(), &st1) != 0) ||
+       (stat(filename2.c_str(), &st2) != 0) ||
        (st1.st_size != st2.st_size))
    {
       return(false);
    }
 
-   if ((fd1 = open(filename1, O_RDONLY)) < 0)
+   if ((fd1 = open(filename1.c_str(), O_RDONLY)) < 0)
    {
       return(false);
    }
-   if ((fd2 = open(filename2, O_RDONLY)) < 0)
+   if ((fd2 = open(filename2.c_str(), O_RDONLY)) < 0)
    {
       close(fd1);
       return(false);
@@ -960,10 +955,10 @@ static void do_source_file(const char *filename_in,
    bool       did_open    = false;
    bool       need_backup = false;
    file_mem   fm;
-   const char *filename_tmp = NULL;
+   string     filename_tmp;
 
    /* Do some simple language detection based on the filename extension */
-   if (cpd.lang_flags == 0)
+   if (!cpd.lang_forced || (cpd.lang_flags == 0))
    {
       cpd.lang_flags = language_from_filename(filename_in);
    }
@@ -991,21 +986,13 @@ static void do_source_file(const char *filename_in,
       {
          /* Create 'outfile.uncrustify' */
          filename_tmp = fix_filename(filename_out);
-         if (filename_tmp == NULL)
-         {
-            LOG_FMT(LERR, "%s: Out of memory\n", __func__);
-            cpd.error_count++;
-            free(fm.data);
-            return;
-         }
 
          if (!no_backup)
          {
-            if (backup_copy_file(filename_in, fm.data, fm.length) != SUCCESS)
+            if (backup_copy_file(filename_in, fm.raw) != SUCCESS)
             {
                LOG_FMT(LERR, "%s: Failed to create backup file for %s\n",
                        __func__, filename_in);
-               free(fm.data);
                cpd.error_count++;
                return;
             }
@@ -1014,18 +1001,12 @@ static void do_source_file(const char *filename_in,
       }
       make_folders(filename_tmp);
 
-      pfout = fopen(filename_tmp, "wb");
+      pfout = fopen(filename_tmp.c_str(), "wb");
       if (pfout == NULL)
       {
          LOG_FMT(LERR, "%s: Unable to create %s: %s (%d)\n",
-                 __func__, filename_tmp, strerror(errno), errno);
+                 __func__, filename_tmp.c_str(), strerror(errno), errno);
          cpd.error_count++;
-         free(fm.data);
-         if ((filename_tmp != NULL) && (filename_tmp != filename_out))
-         {
-            delete [] (char *)filename_tmp;
-            filename_tmp = NULL;
-         }
          return;
       }
       did_open = true;
@@ -1033,9 +1014,7 @@ static void do_source_file(const char *filename_in,
    }
 
    cpd.filename = filename_in;
-   uncrustify_file(fm.data, fm.length, pfout, parsed_file);
-
-   free(fm.data);
+   uncrustify_file(fm, pfout, parsed_file);
 
    if (did_open)
    {
@@ -1046,13 +1025,13 @@ static void do_source_file(const char *filename_in,
          backup_create_md5_file(filename_in);
       }
 
-      if ((filename_tmp != NULL) && (filename_tmp != filename_out))
+      if (filename_tmp != filename_out)
       {
          /* We need to compare and then do a rename */
          if (file_content_matches(filename_tmp, filename_out))
          {
             /* No change - remove tmp file */
-            (void)unlink(filename_tmp);
+            (void)unlink(filename_tmp.c_str());
          }
          else
          {
@@ -1064,15 +1043,13 @@ static void do_source_file(const char *filename_in,
             (void)unlink(filename_out);
 #endif
             /* Change - rename filename_tmp to filename_out */
-            if (rename(filename_tmp, filename_out) != 0)
+            if (rename(filename_tmp.c_str(), filename_out) != 0)
             {
                LOG_FMT(LERR, "%s: Unable to rename '%s' to '%s'\n",
-                       __func__, filename_tmp, filename_out);
+                       __func__, filename_tmp.c_str(), filename_out);
                cpd.error_count++;
             }
          }
-         delete [] (char *)filename_tmp;
-         filename_tmp = NULL;
       }
 
 #ifdef HAVE_UTIME_H
@@ -1092,7 +1069,7 @@ static void add_file_header()
    if (!chunk_is_comment(chunk_get_head()))
    {
       /*TODO: detect the typical #ifndef FOO / #define FOO sequence */
-      tokenize(cpd.file_hdr.data, cpd.file_hdr.length, chunk_get_head());
+      tokenize(cpd.file_hdr.data, chunk_get_head());
    }
 }
 
@@ -1115,7 +1092,7 @@ static void add_file_footer()
          LOG_FMT(LSYS, "Adding a newline at the end of the file\n");
          newline_add_after(pc);
       }
-      tokenize(cpd.file_ftr.data, cpd.file_ftr.length, NULL);
+      tokenize(cpd.file_ftr.data, NULL);
    }
 }
 
@@ -1190,7 +1167,7 @@ static void add_func_header(c_token_t type, file_mem& fm)
       {
          /* Insert between after and ref */
          chunk_t *after = chunk_get_next_ncnl(ref);
-         tokenize(fm.data, fm.length, after);
+         tokenize(fm.data, after);
          for (tmp = chunk_get_next(ref); tmp != after; tmp = chunk_get_next(tmp))
          {
             tmp->level = after->level;
@@ -1200,21 +1177,129 @@ static void add_func_header(c_token_t type, file_mem& fm)
 }
 
 
-static void uncrustify_start(const char *data, int data_len)
+static void add_msg_header(c_token_t type, file_mem& fm)
+{
+   chunk_t *pc;
+   chunk_t *ref;
+   chunk_t *tmp;
+   bool    do_insert;
+
+   for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next_ncnlnp(pc))
+   {
+      if (pc->type != type)
+      {
+         continue;
+      }
+
+      do_insert = false;
+
+      /* On a function proto or def. Back up to a close brace or semicolon on
+       * the same level
+       */
+      ref = pc;
+      while ((ref = chunk_get_prev(ref)) != NULL)
+      {
+         /* ignore the CT_TYPE token that is the result type */
+         if ((ref->level != pc->level) &&
+             ((ref->type == CT_TYPE) ||
+              (ref->type == CT_PTR_TYPE)))
+         {
+            continue;
+         }
+
+         if ((ref->level != pc->level) && (ref->type == CT_OC_CATEGORY))
+         {
+            ref = chunk_get_next_ncnl(ref);
+            if (ref)
+            {
+               do_insert = true;
+            }
+            break;
+         }
+
+         /* Bail if we change level or find an access specifier colon */
+         if ((ref->level != pc->level) || (ref->type == CT_PRIVATE_COLON))
+         {
+            do_insert = true;
+            break;
+         }
+
+         /* If we hit an angle close, back up to the angle open */
+         if (ref->type == CT_ANGLE_CLOSE)
+         {
+            ref = chunk_get_prev_type(ref, CT_ANGLE_OPEN, ref->level, CNAV_PREPROC);
+            continue;
+         }
+
+         /* Bail if we hit a preprocessor and cmt_insert_before_preproc is false */
+         if (ref->flags & PCF_IN_PREPROC)
+         {
+            tmp = chunk_get_prev_type(ref, CT_PREPROC, ref->level);
+            if ((tmp != NULL) && (tmp->parent_type == CT_PP_IF))
+            {
+               tmp = chunk_get_prev_nnl(tmp);
+               if (chunk_is_comment(tmp) &&
+                   !cpd.settings[UO_cmt_insert_before_preproc].b)
+               {
+                  break;
+               }
+            }
+         }
+
+         /* Ignore 'right' comments */
+         if (chunk_is_comment(ref) && chunk_is_newline(chunk_get_prev(ref)))
+         {
+            break;
+         }
+
+         if ((ref->level == pc->level) &&
+             ((ref->flags & PCF_IN_PREPROC) ||
+              (ref->type == CT_SEMICOLON) ||
+              (ref->type == CT_BRACE_CLOSE) ||
+              (ref->type == CT_OC_CLASS)))
+         {
+            do_insert = true;
+            break;
+         }
+      }
+
+      if (do_insert)
+      {
+         /* Insert between after and ref */
+         chunk_t *after = chunk_get_next_ncnl(ref);
+         tokenize(fm.data, after);
+         for (tmp = chunk_get_next(ref); tmp != after; tmp = chunk_get_next(tmp))
+         {
+            tmp->level = after->level;
+         }
+      }
+   }
+}
+
+
+static void uncrustify_start(const deque<int>& data)
 {
    /**
     * Parse the text into chunks
     */
-   tokenize(data, data_len, NULL);
+   tokenize(data, NULL);
+
+   /* Get the column for the fragment indent */
+   if (cpd.frag)
+   {
+      chunk_t *pc = chunk_get_head();
+
+      cpd.frag_cols = (pc != NULL) ? pc->orig_col : 0;
+   }
 
    /* Add the file header */
-   if (cpd.file_hdr.data != NULL)
+   if (cpd.file_hdr.data.size() > 0)
    {
       add_file_header();
    }
 
    /* Add the file footer */
-   if (cpd.file_ftr.data != NULL)
+   if (cpd.file_ftr.data.size() > 0)
    {
       add_file_footer();
    }
@@ -1256,10 +1341,58 @@ static void uncrustify_start(const char *data, int data_len)
 }
 
 
-static void uncrustify_file(const char *data, int data_len, FILE *pfout,
+static void uncrustify_file(const file_mem& fm, FILE *pfout,
                             const char *parsed_file)
 {
-   uncrustify_start(data, data_len);
+   const deque<int>& data = fm.data;
+
+   /* Save off the encoding and whether a BOM is required */
+   cpd.bom = fm.bom;
+   cpd.enc = fm.enc;
+   if (cpd.settings[UO_utf8_force].b ||
+       ((cpd.enc == ENC_BYTE) && cpd.settings[UO_utf8_byte].b))
+   {
+      cpd.enc = ENC_UTF8;
+   }
+   argval_t av;
+   switch (cpd.enc)
+   {
+   case ENC_UTF8:
+      av = cpd.settings[UO_utf8_bom].a;
+      break;
+
+   case ENC_UTF16_LE:
+   case ENC_UTF16_BE:
+      av = AV_FORCE;
+      break;
+
+   default:
+      av = AV_IGNORE;
+      break;
+   }
+   if (av == AV_REMOVE)
+   {
+      cpd.bom = false;
+   }
+   else if (av != AV_IGNORE)
+   {
+      cpd.bom = true;
+   }
+
+   /* Check for embedded 0's (represents a decoding failure or corrupt file) */
+   for (int idx = 0; idx < (int)data.size() - 1; idx++)
+   {
+      if (data[idx] == 0)
+      {
+         LOG_FMT(LERR, "An embedded 0 was found in '%s'.\n", cpd.filename);
+         LOG_FMT(LERR, "The file may be encoded in an unsupported Unicode format.\n");
+         LOG_FMT(LERR, "Aborting.\n");
+         cpd.error_count++;
+         return;
+      }
+   }
+
+   uncrustify_start(data);
 
    /**
     * Done with detection. Do the rest only if the file will go somewhere.
@@ -1270,14 +1403,18 @@ static void uncrustify_file(const char *data, int data_len, FILE *pfout,
       /**
        * Add comments before function defs and classes
        */
-      if (cpd.func_hdr.data != NULL)
+      if (cpd.func_hdr.data.size() > 0)
       {
          add_func_header(CT_FUNC_DEF, cpd.func_hdr);
       }
-      if (cpd.class_hdr.data != NULL)
+      if (cpd.class_hdr.data.size() > 0)
       {
          add_func_header(CT_CLASS, cpd.class_hdr);
       }
+      if (cpd.oc_msg_hdr.data.size() > 0)
+      {
+         add_msg_header(CT_OC_MSG_DECL, cpd.oc_msg_hdr);
+      }
 
       /**
        * Change virtual braces into real braces...
@@ -1304,45 +1441,56 @@ static void uncrustify_file(const char *data, int data_len, FILE *pfout,
       /**
        * Insert line breaks as needed
        */
-      do_blank_lines();
-      newlines_cleanup_braces();
-      if (cpd.settings[UO_nl_after_multiline_comment].b)
+      //do_newlines();
+      int old_changes;
+      cpd.pass_count = 3;
+      do
       {
-         newline_after_multiline_comment();
-      }
-      newlines_insert_blank_lines();
-      if (cpd.settings[UO_pos_bool].tp != TP_IGNORE)
-      {
-         newlines_chunk_pos(CT_BOOL, cpd.settings[UO_pos_bool].tp);
-      }
-      if (cpd.settings[UO_pos_compare].tp != TP_IGNORE)
-      {
-         newlines_chunk_pos(CT_COMPARE, cpd.settings[UO_pos_compare].tp);
-      }
-      if (cpd.settings[UO_pos_conditional].tp != TP_IGNORE)
-      {
-         newlines_chunk_pos(CT_COND_COLON, cpd.settings[UO_pos_conditional].tp);
-         newlines_chunk_pos(CT_QUESTION, cpd.settings[UO_pos_conditional].tp);
-      }
-      if (cpd.settings[UO_pos_comma].tp != TP_IGNORE)
-      {
-         newlines_chunk_pos(CT_COMMA, cpd.settings[UO_pos_comma].tp);
-      }
-      if (cpd.settings[UO_pos_assign].tp != TP_IGNORE)
-      {
-         newlines_chunk_pos(CT_ASSIGN, cpd.settings[UO_pos_assign].tp);
-      }
-      if (cpd.settings[UO_pos_arith].tp != TP_IGNORE)
-      {
-         newlines_chunk_pos(CT_ARITH, cpd.settings[UO_pos_arith].tp);
-      }
-      newlines_class_colon_pos();
-      if (cpd.settings[UO_nl_squeeze_ifdef].b)
-      {
-         newlines_squeeze_ifdef();
-      }
-      newlines_eat_start_end();
-      newlines_cleanup_dup();
+         old_changes = cpd.changes;
+
+         LOG_FMT(LNEWLINE, "Newline loop start: %d\n", cpd.changes);
+
+         newlines_cleanup_dup();
+         newlines_cleanup_braces();
+         if (cpd.settings[UO_nl_after_multiline_comment].b)
+         {
+            newline_after_multiline_comment();
+         }
+         newlines_insert_blank_lines();
+         if (cpd.settings[UO_pos_bool].tp != TP_IGNORE)
+         {
+            newlines_chunk_pos(CT_BOOL, cpd.settings[UO_pos_bool].tp);
+         }
+         if (cpd.settings[UO_pos_compare].tp != TP_IGNORE)
+         {
+            newlines_chunk_pos(CT_COMPARE, cpd.settings[UO_pos_compare].tp);
+         }
+         if (cpd.settings[UO_pos_conditional].tp != TP_IGNORE)
+         {
+            newlines_chunk_pos(CT_COND_COLON, cpd.settings[UO_pos_conditional].tp);
+            newlines_chunk_pos(CT_QUESTION, cpd.settings[UO_pos_conditional].tp);
+         }
+         if (cpd.settings[UO_pos_comma].tp != TP_IGNORE)
+         {
+            newlines_chunk_pos(CT_COMMA, cpd.settings[UO_pos_comma].tp);
+         }
+         if (cpd.settings[UO_pos_assign].tp != TP_IGNORE)
+         {
+            newlines_chunk_pos(CT_ASSIGN, cpd.settings[UO_pos_assign].tp);
+         }
+         if (cpd.settings[UO_pos_arith].tp != TP_IGNORE)
+         {
+            newlines_chunk_pos(CT_ARITH, cpd.settings[UO_pos_arith].tp);
+         }
+         newlines_class_colon_pos();
+         if (cpd.settings[UO_nl_squeeze_ifdef].b)
+         {
+            newlines_squeeze_ifdef();
+         }
+         do_blank_lines();
+         newlines_eat_start_end();
+         newlines_cleanup_dup();
+      } while ((old_changes != cpd.changes) && (cpd.pass_count-- > 0));
 
       mark_comments();
 
@@ -1410,18 +1558,17 @@ static void uncrustify_file(const char *data, int data_len, FILE *pfout,
 
       if (cpd.settings[UO_code_width].n > 0)
       {
-         int max_passes = 3;
-         int prev_changes;
+         cpd.pass_count = 3;
          do
          {
-            prev_changes = cpd.changes;
+            old_changes = cpd.changes;
             do_code_width();
-            if (prev_changes != cpd.changes)
+            if (old_changes != cpd.changes)
             {
                align_all();
                indent_text();
             }
-         } while ((prev_changes != cpd.changes) && (--max_passes > 0));
+         } while ((old_changes != cpd.changes) && (cpd.pass_count-- > 0));
       }
 
       /**
@@ -1469,12 +1616,6 @@ static void uncrustify_end()
       chunk_del(pc);
    }
 
-   if (cpd.bom != NULL)
-   {
-      chunk_del(cpd.bom);
-      cpd.bom = NULL;
-   }
-
    /* Clean up some state variables */
    cpd.unc_off     = false;
    cpd.al_cnt      = 0;
