@@ -14,3 +14,10 @@ void f(){if(bar())baz(1);else baz(2);}
 
 int foo() { return 0; }
 
+void foo(int a, int b)
+{
+if (a == b) { a++; }
+else { b++; }
+if (a == b) a++;
+else b++;
+}
