@@ -62,3 +62,13 @@ int &foo()
    return nuts;
 }
 
+void Foo::bar() {}
+
+Foo::Foo() {}
+
+Foo::~Foo() {}
+
+void func(void)
+{
+Directory dir("arg");
+}
