@@ -71,11 +71,25 @@ void f()
 
 template < class > struct type;
 
-template < class T > 
+template < class T >
 class X {
   typedef type < T > base ;
   void f () {
-    ( base :: operator * () ) ;; 
+    ( base :: operator * () ) ;;
   };
 };
 
+namespace N
+{
+class C
+{
+#define NOP(x) { }
+};
+}
+
+namespace N
+{
+   class C
+   {
+   };
+}
