@@ -1,9 +1,9 @@
 
-#define SOME_VAL1      ((MYINT)-1)
-#define SOME_VAL2      (-2)
-#define SOME_VAL3      -3
-#define MULT(X, Y)      (X)*(Y)
-#define SOME_JUNK      /*lint -e123 */ (const mytype *)-1
+#define SOME_VAL1    ((MYINT)-1)
+#define SOME_VAL2    (-2)
+#define SOME_VAL3    -3
+#define MULT(X, Y)    (X)*(Y)
+#define SOME_JUNK    /*lint -e123 */ (const mytype *)-1
 
 typedef (*my_fcn_ptr_t)(char *, int);
 typedef (my_fcn_t)(char *, int);
@@ -81,5 +81,9 @@ void foo(void)
    a = (int)*(x);
    a = (CHAR)*(x);
    a = (Uint) * (x);
+
+   a = (unsigned int)(1 + 4);
+   a = (int)(1 + 1);
+   a = (void *)(&str);
 }
 
