@@ -17,9 +17,9 @@ static int short_function_name(struct device *dev,
     abc = call_some_other_really_long_function.of_some_sort(
         some_long_parameter1, some_long_parameter2);
 
-    abc.def.ghi = call_some_other_really_long_function.
-                  of_some_sort(some_long_parameter1,
-                               some_long_parameter2);
+    abc.def.ghi =
+        call_some_other_really_long_function.of_some_sort(
+            some_long_parameter1, some_long_parameter2);
 
     abcdefghijklmnopqrstuvwxyz = abc + def + ghi + jkl +
                                  mno + prq + stu + vwx + yz;
