@@ -47,3 +47,28 @@ struct narg {
 	const char *ghi;
 };
 
+void f2(void)
+{
+	{ i++; }
+
+	for (;;) ;
+
+	for (;;) {   }
+}
+
+int main(int argc, char *argv[])
+{
+	if( argc == 1 )
+	{
+		printf("one");
+	}
+	else if( argc == 2 )
+	{
+		printf("two");
+	}
+	else
+	{
+		printf("%d", argc);
+	}
+	return 0;
+}
