@@ -1,28 +1,31 @@
+
 void bla
 (
-)
-{
-}
-
-void bla2
-(
-)
-{
-}
-
+);
 void ble
 (
         int  a,
         char b
-)
-{
-}
-
+);
 void ble2
 (
         int  a,
         char b
-)
+);
+
+
+void bla()
 {
 }
 
+void bla2()
+{
+}
+
+void ble( int a, char b )
+{
+}
+
+void ble2( int a, char b )
+{
+}
