@@ -1,9 +1,8 @@
-template<typename T, template<typename>class SpecialClass>
+template<typename T, template<typename> class SpecialClass>
 class Example
 {
   // Copy constructor with other variants of Example
-  template<template<typename>class OtherSpecialClass>
-  Example(const Example<T, OtherSpecialClass>& other)
+  template<template<typename> class OtherSpecialClass> Example(const Example<T, OtherSpecialClass>& other)
   {
     // do something useful here
   }
