@@ -13,3 +13,11 @@ public:
    }
 };
 
+void test(void)
+{
+   return x != 0
+          && x >= 1
+          && x < 2
+          && y >= 3
+          && y < 4;
+}
