@@ -55,3 +55,16 @@ int &foo()
    return nuts;
 }
 
+void Foo::bar() {
+}
+
+Foo::Foo() {
+}
+
+Foo::~Foo() {
+}
+
+void func(void)
+{
+   Directory dir("arg");
+}
