@@ -47,6 +47,17 @@ int CFoo::operator +()
 {
 }
 
+/**
+ * CLASS:  $(fclass)
+ * METHOD: func
+ * TODO: DESCRIPTION
+ * @return TODO
+ */
+map<string, int> func()
+{
+	// some codes
+}
+
 /**
  * CLASS:  $(fclass)
  * METHOD: some_func
