@@ -20,10 +20,3 @@ int main(int argc, const char *argv[])
 
    return(0);
 }
-
-@interface EmptyClass : NSObject
-@end
-
-@interface EmptyClass : NSObject {}
-@end
-
